module Example.React where

import Data.Maybe (Maybe(..), fromMaybe)
import Data.Nullable (toMaybe, null)
import DOM.Simple as DOM
import DOM.Simple (Element)
import DOM.Simple.Console (log, log2)
import DOM.Simple.Document (document)
import DOM.Simple.Event (class IsEvent)
import DOM.Simple.Event as DE
import Effect (Effect)
import FFI.Simple ((...), delay, args2)
import Math (remainder)
import Reactix as R
import Reactix.DOM.HTML as H
import Reactix.SyntheticEvent as RE
import Toestand as T

import Graphics.D3.Selection as Selection
import Graphics.D3.Selection ((>=>), (>=>-), (>=>++))

import Prelude

main :: Effect Unit
main = paint $ toMaybe (document ... "getElementById" $ [ "react" ])

render :: R.Element -> DOM.Element -> Effect Unit
render e d = delay unit $ \_ -> pure $ R.reactDOM ... "render" $ args2 e d

paint :: Maybe Element -> Effect Unit
paint Nothing = log "[main] container not found"
paint (Just c) = render (app {} []) c

app :: {} -> Array R.Element -> R.Element
app = R.createElement appCpt

appCpt :: R.Component ()
appCpt = R.hooksComponentWithModule "Example.React" "app" cpt where
  cpt {} _ = do
    containerRef <- R.useRef null

    d3Data <- T.useBox [ "hello" ]
    d3Data' <- T.useLive T.unequal d3Data
    newItem <- T.useBox (Nothing :: Maybe String)
    newItem' <- T.useLive T.unequal newItem

    R.useLayoutEffect' $ do
      -- HACK: Initially, R.readRef containerRef returns null (in this
      -- R.useLayoutEffect' call). Using `delay unit` is a hack to wait for the
      -- ref to fill in with proper element.
      delay unit $ \_ -> do
        let mContainer = toMaybe $ R.readRef containerRef
        case mContainer of
          Nothing -> pure unit
          Just cr -> do
            _ <- Selection.rootSelectEl cr
                >>= Selection.selectAll "p"
                >>= Selection.bindData d3Data'
                >>= Selection.enter
                >=>++ "p"
                >>= Selection.style'' "color" (\_d idx -> if idx `remainder` 2.0 == 0.0 then "green" else "red")
                >>= Selection.text' show
            pure unit

    pure $ H.div {}
      [ H.div {}
        [ H.form {}
          [ H.label {} [ H.text "Enter new item" ]
          , H.input { defaultValue: fromMaybe "" newItem'
                    , on: { change: onInputChange newItem } }
          , H.button { on: { click: onSubmitClick d3Data newItem newItem' } } [ H.text "submit" ] ]
        ]
      , H.div { ref: containerRef
              , width: "400"
              , height: "200" } []
      ]
    where
      onInputChange newItem e = do
        let val = R.unsafeEventValue e
        T.write_ (Just val) newItem
      onSubmitClick :: T.Box (Array String) -> T.Box (Maybe String) -> Maybe String -> RE.SyntheticEvent DE.MouseEvent -> Effect Unit
      onSubmitClick d3Data newItem newItem' e = do
        RE.preventDefault e
        RE.stopPropagation e
        case newItem' of
          Nothing -> pure unit
          Just ni -> T.modify_ (\d -> d <> [ ni ]) d3Data
        T.write_ Nothing newItem
